/* rexx program to synchronize two trees */
/* Requires Classic Rexx, Rexxutil
/* Requires MySQL server and  RexxSQL <http://sourceforge.net/projects/rexxsql/>    */
/* Requires rexxbos <http://www.logisoftar.com >    */


dbusername='mwc'
dbpasswd='snarkle'
dbserver='192.168.1.2'
dbname='mysql'


/* -----  no changes past this point ----------------------------- */

call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

Call RxFuncAdd 'SQLLoadFuncs', 'rexxsql', 'SQLLoadFuncs'
Call SQLLoadFuncs

/* Load REXXBOS functions and make them available  */
call RxFuncAdd 'RxLoadFuncs', 'REXXBOS', 'RxLoadFuncs'
call RxLoadFuncs
 
call syscls
say 'TreeSync .02'
say ''
starttime=time('N')
startdate=date('U')

ARG Parm1 Parm2 

if Parm1 = '' then 
 signal syntax

sourcedir = Parm1 /* input path */
destdir = Parm2 /* Output path */

if right(sourcedir,1) <> '\' then
sourcedir=sourcedir'\'

if right(destdir,1) <> '\' then
destdir=destdir'\'

/* --------------------------------------------------------------------------------------- */

/* check for existance of source tree */
w=sourcedir'*'
rc = SysFileTree(w, source, 'B') 
if rc !=0 then do
        say 'Problem processing source tree - exiting'
        say 'Press <enter> to continue'
        pull dummy
        signal abort
end /* do */
if source.0=0 then do
        say 'No files or subdirs found in source tree - exiting'
        say 'Press <enter> to continue'
        pull dummy
        signal abort
end /* do */


/* --------------------------------------------------------------------------------------- */


say 'Connect to the database'
If SQLConnect( 'c1', dbusername, dbpasswd, dbname ,dbserver) < 0 Then Abort( 'connecting' )

/* If SQLPrepare('dropTS', 'drop database treesync') < 0 Then Abort( 'dropTD' ) */

If SQLPrepare( 'createdb', 'create database if not exists treesync') < 0 Then Abort( 'createdb' ) 
If SQLPrepare( 'usedb', 'use treesync') < 0 Then Abort( 'usedb' ) 

If SQLPrepare( 'createStbl', 'create table if not exists source (sfile varchar(255) not null primary key, ssig char(35) not null)') < 0 Then Abort( 'createStbl' ) 
If SQLPrepare( 'createDtbl', 'create table if not exists dest (dfile varchar(255) not null primary key, dsig char(35) not null)') < 0 Then Abort( 'createDtbl' ) 
If SQLPrepare( 'createTtbl', 'create table if not exists tree (name varchar(255) not null primary key)') < 0 Then Abort( 'createTtbl' ) 

/* source tree ------------------------------------------------------------ */

w=sourcedir'*'
say 'SysFileTree 'w
etime=time('R')
rc = SysFileTree(w, source, 'FST')   /* T = Date&Time  YY/MM/DD/HH/MM */
say 'Count: 'source.0'    Elapsed Time: 'time('E')    
say ''
say 'parse & insert to db table'            
etime=time('R')
do t=1 to source.0 
    parse var source.t DT size attr filename
    sig=dt''size
    filename=strip(filename,'L')
    filename=translate(filename,'/','\')
    w=length(sourcedir)
    filename=right(filename,length(filename)-w)
    vals='insert into source(sfile,ssig) values ("'filename'","'sig'")'
    If SQLPrepare( 'insertS', vals) < 0 Then Abort( 'insertS' )  
end /* do */
drop source.
say 'Elapsed Time: 'time('E')    
say ''

/* destination tree ----------------------------------------------------- */

w=destdir'*'
say 'SysFileTree 'w 
etime=time('R')
rc = SysFileTree(w, dest, 'FST',,'**---')   /* T = Date&Time  YY/MM/DD/HH/MM */
say 'Count: 'dest.0'    Elapsed Time: 'time('E')    
say ''
say 'parse & insert to db table'            
etime=time('R')
do t=1 to dest.0 
    parse var dest.t DT size attr filename
    sig=dt''size
    filename=strip(filename,'L')
    filename=translate(filename,'/','\')
    w=length(destdir)
    filename=right(filename,length(filename)-w)
    vals='insert into dest(dfile,dsig) values ("'filename'","'sig'")'
    If SQLPrepare( 'insertD', vals) < 0 Then Abort( 'insertD' )  
end /* do */
drop dest.
say 'Elapsed Time: 'time('E')    
say ''
say 'replicate tree'
etime=time('R')
w=sourcedir'*'
rc = SysFileTree(w, stree, 'DSO')
do t=1 to stree.0 
    w=length(sourcedir)
    dpath=destdir''right(stree.t,length(stree.t)-w)
    vals='insert into tree(name) values ("'dpath'")'
    If SQLPrepare( 'insertT', vals) < 0 Then Abort( 'insertT' )  
    rc=RbCreateDir(dpath)
end /* do */
say 'Count: 'stree.0  'Elapsed Time: 'time('E')    
drop stree.
say ''
say 'prune tree'
etime=time('R')
w=destdir'*'
rc = SysFileTree(w, dtree, 'DSO')
do t=1 to dtree.0 
     /* select * from tree where name = dtree.t */
     /* if not found, delete the subdir path */
     sql='select * from tree where name = "'dtree.t'"'
     rc = sqlprepare(t1,sql)
     rc = sqlopen(t1)
     rc=sqlfetch(t1)
     if rc = 0 then call RbDelTree(dtree.t)
     rc = sqldispose(t1)
end /* do */
say 'Elapsed Time: 'time('E')    
say ''
/* now we sync the files */
/* if the dest record doesn't exist, copy the file */

say 'Finding and copying NEW files'
etime=time('R')
sql='select source.sfile from source left join dest on source.sfile=dest.dfile where dest.dfile is null'
rc = sqlprepare(t1,sql)
rc = sqlopen(t1)
rc=sqlfetch(t1)
count=0
do forever
        rc=sqlfetch(t1)
        if rc=0 then leave
        if rc <0 then abort()
        rc=sqlgetdata(t1,sfile)
        s=sourcedir''t1.sfile
        d=destdir''t1.sfile
        s=translate(s,'\','/')
        d=translate(d,'\','/')
        rc=RbCopy(s,d,'X')
        if rc <> 0 then do
               say 'copy of 's' failed!  Check destination drive & path'
               signal abort
        end /* do */
        count=count+1
        say s d
end /* do */
rc = sqldispose(t1)
say 'Count: 'count  'Elapsed Time: 'time('E')    
say ''

/* delete files that exist on dest but not source */
say 'Finding and deleting unmatched files'
etime=time('R')
sql='select dest.dfile from dest left join source on dest.dfile=source.sfile where source.sfile is null'
rc = sqlprepare(t1,sql)
rc = sqlopen(t1)
rc=sqlfetch(t1)
count=0
do forever
        rc=sqlfetch(t1)
        if rc=0 then leave
        if rc <0 then abort()
        rc=sqlgetdata(t1,dfile)
        d=destdir''t1.dfile
        d=translate(d,'\','/')
        rc=Rbdelete(d)  
               if rc <> 0 then do
               say 'delete of 'd' failed!'
        end /* do */
        count=count+1
end /* do */
rc = sqldispose(t1)
say 'Count: 'count  'Elapsed Time: 'time('E')    

/* and if the sigs don't match, copy the file */

/*  select * from source left join dest on source.sfile=dest.dfile where source.ssig<>dest.dsig  */
say ''
say 'Finding and copying changed files'
etime=time('R')
sql='select * from source left join dest on source.sfile=dest.dfile where source.ssig<>dest.dsig'
rc = sqlprepare(t1,sql)
rc = sqlopen(t1)
rc=sqlfetch(t1)
count=0
say 'Copying...'
do forever
        rc=sqlfetch(t1)
        if rc=0 then leave
        if rc <0 then abort()
        rc=sqlgetdata(t1,sfile)
        s=sourcedir''t1.sfile
        d=destdir''t1.sfile
        s=translate(s,'\','/')
        d=translate(d,'\','/')
        rc=RbCopy(s,d,'X')
        count=count+1
        say s d
        if rc <> 0 then do
               say 'copy of 's' failed!  Check destination drive & path'
               signal abort
        end /* do */
end /* do */
rc = sqldispose(t1)
say 'Count: 'count  'Elapsed Time: 'time('E')    
say ''

endtime=time('N')
enddate=date('U')

say 'Started: 'startdate    starttime
say 'ended: ' enddate      endtime

/* done */
abort:
If SQLPrepare('dropTS', 'drop database treesync') < 0 Then Abort( 'dropTD' )
If SQLDisconnect( 'c1' ) < 0 Then Abort( 'disconnecting' ) 
Say 'Disconnected from database'

Call SQLDropFuncs
call SysdropFuncs
call RxdropFuncs
exit

Syntax:
call syscls
say ''
say ''
say 'treesync .02'
say ''

say 'Syntax:'
say 'treesync srcdir destdir'
say ''
say "Basis for 'duplicate/not duplicate' test is file date, time, and size.  no CRC or MD5 is calculated."
say ''
exit


/*
Abort: Procedure Expose sqlca.
Parse Arg msg
Say 'Program failed:' msg
Say sqlca.interrm
If Datatype( sqlca.sqlerrm.0 ) = 'NUM' Then
   Do i = 1 To sqlca.sqlerrm.0
      Say '('sqlca.sqlstate.i')' sqlca.sqlerrm.i
   End
Else
   Do
      Say '('sqlca.sqlstate')' sqlca.sqlerrm
   End
Call SQLDropFuncs 'UNLOAD'
Exit 1
*/

